package vishnu.cluster;

import java.util.*;


public class Clustering
{
    public static final int COMPLETE_LINKAGE = 0;
    public static final int AVERAGE_LINKAGE  = 1;
    public static final int SINGLE_LINKAGE   = 2;
    public static final int BUCKSHOT = 0;
    public static final int COMPLETE = 1;
    
    private Vector [] clusters;
    private double [][] centroids;
    
    public Vector [] getClusters()
    {
        return clusters;
    }

    public double[][] getCentroids()
    {
        return centroids;
    }
    
    
    public Clustering (int linkage, float[][] KDMatrix, int min_c_items, int target_clusters, int limit)
    {
        SimMatrix sim_matrix;
	LinkFunc linkage_func=null;
	
        int wordCount = KDMatrix[0].length;
	int hitDocs = KDMatrix.length;
        
	Cluster toplevel_cluster;
        ClusterArray splitclusters = null;
	int[] n_clusters = new int[]{0};
	
        double threshold;
	
        Distance sim_func = new EuclideanDistance();
	
	double[] heights;

        int[] coding = new int[hitDocs];
        

        for (int c = 0; c < hitDocs; c++) coding[c] = c;
	
	switch (linkage){
	case COMPLETE_LINKAGE:
	    linkage_func = new MaxLink();
	    break;
	case AVERAGE_LINKAGE:
	    linkage_func = new AveLink();
	    break;
	case SINGLE_LINKAGE:
	    linkage_func = new MinLink();
	    break;
	default:
	    System.exit(0);
	}
	

        /**** make sure upper bound does not exceed #docs ****/
        
	if(limit > hitDocs) limit = hitDocs;
	limit = (int)hitDocs/2;
	

	/**** get similarity matrix of the first limit docs with each other ****/
	
        sim_matrix = new SimMatrix(KDMatrix, limit, sim_func);
	
	heights = new double[hitDocs];
	
        Hierarchical hierarchical = new Hierarchical(limit);
        
        toplevel_cluster = hierarchical.clustering(sim_matrix, limit, heights, linkage_func);
	if( toplevel_cluster == null )
	    return;
	
        System.out.println("Clustered items: " + toplevel_cluster.items);
	Cluster c1 = toplevel_cluster.child_1;
	Cluster c2 = toplevel_cluster.child_2;
	System.out.println("Child1: " + c1.items);
	System.out.println("Child2: " + c2.items);
	
	splitclusters = new ClusterArray(0);
        
	threshold = splitclusters.split(toplevel_cluster, heights, limit, target_clusters, min_c_items, n_clusters);
	
    	System.out.println(n_clusters[0] + " clusters");
	System.out.println("Threshold: " + threshold);
	
        System.out.println("Target_clusters " + target_clusters);
	System.out.println("Limit " + limit);
	System.out.println("#docs: " + hitDocs);
	
        Iterative iter = new Iterative(wordCount);
	
	iter.clustering(splitclusters, n_clusters, KDMatrix, limit, hitDocs, sim_func);
	
	System.out.println("Finished iterative clustering");

        clusters = new Vector[n_clusters[0]];
        centroids = new double[n_clusters[0]][wordCount];

	for (int i = 0; i < n_clusters[0]; i++){
	    clusters[i]=new Vector();
	    Cluster clust = (Cluster)splitclusters.contents.elementAt(i);
	    clust.addChildrenToOutputVector(coding,clusters[i]);
	    
	    for (int j = 0; j < clust.iter_items_num; j++){
		Integer ii = (Integer)clust.iter_items.elementAt(j);
		Integer jj = new Integer (coding[ii.intValue()]);
		clusters[i].addElement(jj);
	    }
	    
	    float [] centroid = clust.getCentroid();
	    
	    for (int j = 0; j < wordCount; j++){
		centroids[i][j]=centroid[j];
	    }
	}
    }
}
